#define UARTF

#if ARDUINO < 100
  #include <WProgram.h>
#else
    #include <Arduino.h>
#endif
#include  <SoftwareSerial.h>

#define ACK_SUCCESS 		0x00
#define ACK_FAIL		  	0x01
#define ACK_FULL		  	0x04
#define ACK_NOUSER			0x05
#define ACK_USER_EXIST	0x06
#define ACK_FIN_EXIST		0x07
#define ACK_TIMEOUT			0x08

#define CMD_SLEEP		  	0x2c
#define CMD_MODE		  	0x2d
#define CMD_ADD1		  	0x01
#define CMD_ADD2		  	0x02
#define CMD_ADD3		  	0x03
#define CMD_DEL_USER		0x04
#define CMD_D_ALL_USER	0x05
#define CMD_CNT_USER		0x09
#define CMD_COMP_1_1  	0x0b
#define CMD_COMP_I_N		0x0c
#define CMD_USER_PRIV		0x0A
#define CMD_GET_VER			0x26
#define CMD_COMP_LV			0x28
#define CMD_UPLOAD			0x24
#define CMD_UPLOAD_EX	  0x03
#define UART_BAUD       19200

class MECHA_UARTFinger {
  public:

    MECHA_UARTFinger(SoftwareSerial *);

    MECHA_UARTFinger(HardwareSerial *);
    
    int begin();
    
    void Sleep();
    
    void Mode(byte value);
    
    int AddUser1(uint16_t id,byte privilege);
    
    int AddUser2(uint16_t id,byte privilege);
    
    int AddUser3(uint16_t id,byte privilege);
    
    int DelUser();
    
    int DelUser(uint16_t id);
    
    void CountUser();
    
    int Compare(uint16_t id);
    
    int Compare();
    
    int GetPrivilege(uint16_t id);

    void GetRespon();
    
    uint16_t GetPacket();
    
    byte GetFeedback();
    
  private:
    byte FORMAT[8] = {0xf5,0x00,0x00,0x00,0x00,0x00,0x00,0xf5};
    
    SoftwareSerial *sofSeri = NULL;
    HardwareSerial *hwSeri = NULL;
    Stream *Seri;
    int send(byte *);
    byte DATA[8] = {0,};
};





MECHA_UARTFinger::MECHA_UARTFinger(SoftwareSerial *Serial){
  sofSeri = Serial;
  Seri    = sofSeri;
}

MECHA_UARTFinger::MECHA_UARTFinger(HardwareSerial *Serial){
  hwSeri  = Serial;
  Seri    = hwSeri;
}

int MECHA_UARTFinger::begin(){
  if(hwSeri){hwSeri->begin(UART_BAUD); return 0;}
  if(sofSeri){sofSeri->begin(UART_BAUD); return 0;}
  return -1;
}

void MECHA_UARTFinger::Sleep(){
  byte CMD[8];
  for(int i = 0; i < 8; i++){
    CMD[i] = FORMAT[i];
  }
  CMD[1] = CMD_SLEEP;
  send(CMD);
}

void MECHA_UARTFinger::Mode(byte value){
  byte CMD[8];
  for(int i = 0; i < 8; i++){
    CMD[i] = FORMAT[i];
  }
  CMD[1] = CMD_MODE;
  CMD[4] = value;
  send(CMD);
}

int MECHA_UARTFinger::AddUser1(uint16_t id,byte privilege){
  if(id>0xfff) return -1;
  byte idL = (id & 0xff);
  byte idH = (id >> 16) & 0xff;
  byte CMD[8];
  for(int i = 0; i < 8; i++){
    CMD[i] = FORMAT[i];
  }
  CMD[1] = CMD_ADD1;
  CMD[2] = idH;
  CMD[3] = idL;
  CMD[4] = privilege;
  send(CMD);
}

int MECHA_UARTFinger::AddUser2(uint16_t id,byte privilege){
  if(id>0xfff) return -1;
  byte idL = (id & 0xff);
  byte idH = (id >> 16) & 0xff;
  byte CMD[8];
  for(int i = 0; i < 8; i++){
    CMD[i] = FORMAT[i];
  }
  CMD[1] = CMD_ADD2;
  CMD[2] = idH;
  CMD[3] = idL;
  CMD[4] = privilege;
  send(CMD);
  return 0;
}

int MECHA_UARTFinger::AddUser3(uint16_t id,byte privilege){
  if(id>0xfff) return -1;
  byte idL = (id & 0xff);
  byte idH = (id >> 16) & 0xff;
  byte CMD[8];
  for(int i = 0; i < 8; i++){
    CMD[i] = FORMAT[i];
  }
  CMD[1] = CMD_ADD3;
  CMD[2] = idH;
  CMD[3] = idL;
  CMD[4] = privilege;
  send(CMD);
  return 0;
}

int MECHA_UARTFinger::DelUser(){
  byte CMD[8];
  for(int i = 0; i < 8; i++){
    CMD[i] = FORMAT[i];
  }
  CMD[1] = CMD_D_ALL_USER;
  send(CMD);
  return 0;
}

int MECHA_UARTFinger::DelUser(uint16_t id){
  byte CMD[8];
  for(int i = 0; i < 8; i++){
    CMD[i] = FORMAT[i];
  }
  CMD[1] = CMD_DEL_USER;
  if(id>0xfff) return -1;
  byte idL = (id & 0xff);
  byte idH = (id >> 16) & 0x0f;
  CMD[2] = idH;
  CMD[3] = idL;
  send(CMD);
  return 0;
}

void MECHA_UARTFinger::CountUser(){
  byte CMD[8];
  for(int i = 0; i < 8; i++){
    CMD[i] = FORMAT[i];
  }
  CMD[1] = CMD_CNT_USER;
  send(CMD);
}

int MECHA_UARTFinger::Compare(uint16_t id){
  byte CMD[8];
  for(int i = 0; i < 8; i++){
    CMD[i] = FORMAT[i];
  }
  CMD[1] = CMD_COMP_1_1;
  if(id>0xfff) return -1;
  byte idL = (id & 0xff);
  byte idH = (id >> 16) & 0x0f;
  CMD[2] = idH;
  CMD[3] = idL;
  send(CMD);
  return 0;
}

int MECHA_UARTFinger::Compare(){
  byte CMD[8];
  for(int i = 0; i < 8; i++){
    CMD[i] = FORMAT[i];
  }
  CMD[1] = CMD_COMP_I_N;
  send(CMD);
  return 0;
}

void MECHA_UARTFinger::GetRespon(){
  int cnt = 0;
  while(cnt < 8){
    if(Seri->available()){
      DATA[cnt++] = Seri->read();
    }
  }
  return;
}


uint16_t MECHA_UARTFinger::GetPacket(){
  return ((DATA[2]<<16)|(DATA[3]));
}

byte MECHA_UARTFinger::GetFeedback(){
  return DATA[4];
}


int MECHA_UARTFinger::GetPrivilege(uint16_t id){
  byte CMD[8];
  for(int i = 0; i<8;i++){
    CMD[i] = FORMAT[i];
  }
  CMD[1] = CMD_USER_PRIV;
  if(id>0xfff) return -1;
  byte idL = (id & 0xff);
  byte idH = (id >> 8) & 0x0f;
  CMD[2] = idH;
  CMD[3] = idL;
  send(CMD);
  return 0;
}

int MECHA_UARTFinger::send(byte *data){
  byte cheker = 0x00;
  for(int i = 1; i < 6; i++){
    cheker ^= data[i];
  }
  data[6] = cheker;
  for(int i = 0;i < 8;i++)
  {
    Seri->write(data[i]);
  }
}



#include <SoftwareSerial.h>
#define NEW_USER_ID 1
#define NEW_USER_PRIV 3
SoftwareSerial sofSeri(2,3);
MECHA_UARTFinger fin = MECHA_UARTFinger(&sofSeri);
void setup() {
  Serial.begin(9600);
  fin.begin();
  fin.AddUser1(NEW_USER_ID,NEW_USER_PRIV);
  fin.GetRespon();
  if(ACK_SUCCESS == fin.GetFeedback()){
    Serial.println("Success add user 1/3");
  }
  fin.AddUser2(NEW_USER_ID,NEW_USER_PRIV);
  fin.GetRespon();
  if(ACK_SUCCESS == fin.GetFeedback()){
    Serial.println("Success add user 2/3");
  }
  fin.AddUser3(NEW_USER_ID,NEW_USER_PRIV);
  fin.GetRespon();
  if(ACK_SUCCESS == fin.GetFeedback()){
    Serial.println("Success add user 3/3");
  }
}

void loop() {
  delay(5000);
  fin.Compare();
  fin.GetRespon(); 
  if(ACK_NOUSER != fin.GetFeedback()){
    Serial.print("USER ID: ");
    Serial.println(fin.GetPacket());
  }else {
    Serial.print("ERROR Feedback is: ");
    Serial.println(fin.GetFeedback(),HEX);
  }
}